% To address the new requirements and refine the MATLAB simulation to include:

% The variation of zeta as a function of x for t=z0×θ.
% Compute and plot the electric field for each type of source
%  (a pulse and a Gaussian) for three different values of zeta, making a total of six simulations.
% The ζ factor is typically used to control the time-step relative to the spatial step, 
% ensuring stability in the FDTD method. If ζ varies across the spatial domain, 
% this introduces a non-uniform time-stepping across the grid, 
% which can be complex to implement. For simplicity,
 % we can interpret this requirement as needing to visualize how the electric field develops
 % at t=z0×θ for different ζ values. However, if ζ needs to vary with x dynamically, 
% additional adjustments would be necessary.


% Parameters
c = 1; % speed of light (normalized)
theta = 1; % width of Gaussian pulse
t0 = 5 * theta; % delay of Gaussian pulse
xmax = 20; % spatial domain max
zeta_values = [1, 0.99, 0.5]; % stability factors
deltax = 1; % spatial resolution (grid spacing)
Nx = round(xmax / deltax); % number of spatial points

% Time step is calculated based on the smallest zeta for stability
deltat = min(zeta_values) * deltax / c;
timesteps = round(t0 / deltat); % total time steps, adjusted for the pulse delay

% Source initialization
source_position = 1; % index where source is injected

% Function handles for the sources
pulse_source = @(t) double(t >= t0 - theta && t <= t0 + theta);
gaussian_source = @(t) (1 / sqrt(2 * pi * theta)) * exp(-(t - t0)^2 / (2 * theta^2));

% Simulation for each zeta and source type
for zeta = zeta_values
    for source_type = {'pulse', 'gaussian'}
        % Initialize fields for each run
        Ez = zeros(Nx, 1);
        Hy = zeros(Nx, 1); % Hy field, last element will not be used
        
        % Main FDTD loop
        for T = 1:timesteps
            % Update magnetic field Hy
            for nx = 1:Nx-1
                Hy(nx) = Hy(nx) + (zeta * deltax / c) * (Ez(nx+1) - Ez(nx)) / deltax;
            end

            % Update electric field Ez
            Ez(2:Nx-1) = Ez(2:Nx-1) + (zeta * deltax / c) * (Hy(2:Nx-1) - Hy(1:Nx-2)) / deltax;
            
            % Inject source at x = 0
            if strcmp(source_type, 'pulse')
                Ez(source_position) = Ez(source_position) + pulse_source(T * deltat);
            else
                Ez(source_position) = Ez(source_position) + gaussian_source(T * deltat);
            end
        end
        
        % Plot results for Ez field at time t = z0 * theta
        figure;
        plot((0:Nx-1) * deltax, Ez, 'LineWidth', 2);
        title(['Ez field, ', char(source_type), ', zeta = ', num2str(zeta)]);
        xlabel('x (spatial grid)');
        ylabel('Ez (Electric field)');
        grid on; % Adding grid for better readability
        drawnow; % Draw the plot immediately
    end
end
