
% To explain the differences in the electric field for the pulse (a) and Gaussian (b) sources and for different values of zeta, as well as to analyze ripples, distortions, and how to improve the simulation, here's a comprehensive explanation:

% Differences in the Electric Fields for Sources a and b:
% Pulse Source (a): This is a simple rectangular pulse. The electric field is excited to a value of 1 for a short duration. This kind of source introduces a broad spectrum of frequencies because of its abrupt start and stop, which can cause reflections and standing wave patterns within the simulation domain, especially if the boundaries are not absorbing. The field can also become distorted if the simulation grid is not fine enough to capture the sharp transitions.
% Gaussian Source (b): The Gaussian source smoothly varies in time, which means it has a narrower frequency spectrum compared to the pulse source. It is less likely to cause reflections and standing wave patterns but can suffer from dispersion if the simulation grid does not accurately capture the waveform, which can occur if the grid is too coarse or if the time step is too large.
% Effect of Different zeta Values:
% zeta = 1: This condition, also known as the Courant condition, is the critical point for stability in the FDTD method. It allows for the maximum time step size without causing instability. However, being at the edge of stability can sometimes introduce numerical dispersion where the wave speed appears to change.
% zeta < 1: This condition provides a smaller time step, which can lead to more accurate simulations by reducing numerical dispersion and allowing the waveform to be captured more accurately. However, this also increases the computational load since more time steps are needed.
% Ripples and Distortions: These can be caused by several factors, such as numerical dispersion, reflections from boundaries, the source not being band-limited, or the grid not being fine enough. Ripples indicate that the simulation is capturing multiple frequencies and that these are interfering with each other.
% Improvements:
% Absorbing Boundary Conditions: Implementing Perfectly Matched Layers (PML) or other absorbing boundary conditions can help reduce reflections from the boundaries.
% Grid Refinement: Reducing deltax can improve the spatial resolution and allow the simulation to capture the waveforms more accurately.
% Source Conditioning: Applying a smoother turn-on for the source can help reduce the introduction of high-frequency components that are difficult to simulate accurately.


% Constants
c = 1; % Speed of light normalized to 1
theta = 1; % Width of the Gaussian pulse
t0 = 5 * theta; % Delay of the Gaussian pulse
xmax = 200; % Spatial domain maximum
zeta_values = [1, 0.99, 0.5]; % Array of zeta values for stability factor
deltax_values = [0.1, 0.5, 1, 2]; % Array of deltax values to test

% Preallocate array to store errors for each deltax
error_results = zeros(length(deltax_values), 1);

% Loop over each deltax value
for idx = 1:length(deltax_values)
    deltax = deltax_values(idx);
    deltat = zeta_values(1) * deltax / c; % Smallest zeta for stability
    Nx = round(xmax / deltax); % Number of spatial points

    % Source position
    source_position = 1; % Index where source is injected

    % Initialize fields
    Ez = zeros(Nx, 1);
    Hy = zeros(Nx, 1);

    % Calculate total simulation time for t = z0 * theta
    z0 = 1; % Given in the problem
    simulation_time = z0 * theta;
    Nt = round(simulation_time / deltat);

    % Time-dependent Gaussian source function
    gaussian_source = @(n, dt) (1/sqrt(2*pi*theta)) * exp(-(n*dt-t0)^2 / (2*theta^2));

    % FDTD simulation main loop
    for n = 1:Nt
        % Update Hy field
        for k = 1:Nx-1
            Hy(k) = Hy(k) + (deltat/(c*deltax))*(Ez(k+1) - Ez(k));
        end

        % Update Ez field
        for k = 2:Nx
            Ez(k) = Ez(k) + (deltat/(c*deltax))*(Hy(k) - Hy(k-1));
        end

        % Inject source
        Ez(source_position) = Ez(source_position) + gaussian_source(n, deltat);
    end

    % Analytical solution for each spatial point at final time step
    analytical_solution = arrayfun(@(x) gaussian_source(Nt, deltat - x/c), (0:Nx-1)*deltax);

    % Compute error
    error_numer = sum((Ez - analytical_solution').^2);
    error_denom = sum(analytical_solution.^2);
    error_results(idx) = error_numer / error_denom;
end

% Display results
disp('Errors for different deltax values:');
for i = 1:length(deltax_values)
    fprintf('deltax = %.2f, Error = %.5f\n', deltax_values(i), error_results(i));
end
